@@ -4,6 +4,9 @@ pub mod tree;
 
 // mod human_format;
 use chrono::prelude::*;
+// use clap::AppSettings;
+use clap::ArgEnum;
+use clap::Parser;
 use core::convert::TryFrom;
 use itertools::Itertools;
 use k8s_openapi::api::core::v1::{Node, Pod};
@@ -13,9 +16,6 @@ use prettytable::{cell, format, row, Cell, Row, Table};
 use qty::Qty;
 use std::collections::BTreeMap;
 use std::str::FromStr;
-use structopt::clap::arg_enum;
-use structopt::clap::AppSettings;
-use structopt::StructOpt;
 use tracing::{info, instrument, warn};
 
 #[derive(thiserror::Error, Debug)]
@@ -47,6 +47,18 @@ pub enum Error {
         context: String,
         source: kube::Error,
     },
+
+    #[error("Failed to {context}")]
+    KubeConfigError {
+        context: String,
+        source: kube::config::KubeconfigError,
+    },
+
+    #[error("Failed to {context}")]
+    KubeInferConfigError {
+        context: String,
+        source: kube::config::InferConfigError,
+    },
 }
 
 #[derive(Debug, Clone, Default)]
@@ -315,7 +327,7 @@ pub async fn collect_from_pods(
     namespace: &Option<String>,
 ) -> Result<(), Error> {
     let api_pods: Api<Pod> = if let Some(ns) = namespace {
-        Api::namespaced(client, &ns)
+        Api::namespaced(client, ns)
     } else {
         Api::all(client)
     };
@@ -376,8 +388,8 @@ pub async fn extract_allocatable_from_pods(
         }
         // handler overhead (add to both requests and limits)
         if let Some(ref overhead) = spec.and_then(|s| s.overhead.clone()) {
-            process_resources(&mut resource_requests, &overhead, std::ops::Add::add)?;
-            process_resources(&mut resource_limits, &overhead, std::ops::Add::add)?;
+            process_resources(&mut resource_requests, overhead, std::ops::Add::add)?;
+            process_resources(&mut resource_limits, overhead, std::ops::Add::add)?;
         }
         // push these onto resources
         push_resources(
@@ -490,15 +502,13 @@ pub async fn extract_utilizations_from_pod_metrics(
     Ok(())
 }
 
-arg_enum! {
-    #[derive(Debug, Eq, PartialEq)]
-    #[allow(non_camel_case_types)]
-    pub enum GroupBy {
-        resource,
-        node,
-        pod,
-        namespace,
-    }
+#[derive(Debug, Eq, PartialEq, ArgEnum, Clone)]
+#[allow(non_camel_case_types)]
+pub enum GroupBy {
+    resource,
+    node,
+    pod,
+    namespace,
 }
 
 impl GroupBy {
@@ -532,47 +542,51 @@ impl GroupBy {
     }
 }
 
-arg_enum! {
-    #[derive(Debug, Eq, PartialEq)]
-    #[allow(non_camel_case_types)]
-    pub enum Output {
-        table,
-        csv,
+impl std::fmt::Display for GroupBy {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.write_str(&self.to_string())
     }
 }
 
-#[derive(StructOpt, Debug)]
-#[structopt(
-    global_settings(&[AppSettings::ColoredHelp, AppSettings::VersionlessSubcommands]),
-    author = env!("CARGO_PKG_HOMEPAGE"), about
+#[derive(Debug, Eq, PartialEq, ArgEnum, Clone)]
+#[allow(non_camel_case_types)]
+pub enum Output {
+    table,
+    csv,
+}
+
+#[derive(Parser, Debug)]
+#[clap(
+    // global_settings(&[AppSettings::ColoredHelp, AppSettings::VersionlessSubcommands]),
+    author = env!("CARGO_PKG_HOMEPAGE"), about, version
 )]
 pub struct CliOpts {
     /// The name of the kubeconfig context to use
-    #[structopt(long)]
+    #[clap(long)]
     pub context: Option<String>,
 
     /// Show only pods from this namespace
-    #[structopt(short, long)]
+    #[clap(short, long)]
     pub namespace: Option<String>,
 
     /// Force to retrieve utilization (for cpu and memory), require to have metrics-server https://github.com/kubernetes-sigs/metrics-server
-    #[structopt(short = "u", long)]
+    #[clap(short = 'u', long)]
     pub utilization: bool,
 
     /// Show lines with zero requested and zero limit and zero allocatable
-    #[structopt(short = "z", long)]
+    #[clap(short = 'z', long)]
     pub show_zero: bool,
 
     /// Filter resources shown by name(s), by default all resources are listed
-    #[structopt(short, long)]
+    #[clap(short, long)]
     pub resource_name: Vec<String>,
 
     /// Group information hierarchically (default: -g resource -g node -g pod)
-    #[structopt(short, long, possible_values = &GroupBy::variants(), case_insensitive = true)]
+    #[clap(short, long, arg_enum, ignore_case = true)]
     pub group_by: Vec<GroupBy>,
 
     /// Output format
-    #[structopt(short, long, possible_values = &Output::variants(), case_insensitive = true, default_value = "table")]
+    #[clap(short, long, arg_enum, ignore_case = true, default_value = "table")]
     pub output: Output,
 }
 
@@ -607,13 +621,13 @@ pub async fn new_client(cli_opts: &CliOpts) -> Result<kube::Client, Error> {
             ..Default::default()
         })
         .await
-        .map_err(|source| Error::KubeError {
+        .map_err(|source| Error::KubeConfigError {
             context: "create the kube client config".to_string(),
             source,
         })?,
         None => kube::Config::infer()
             .await
-            .map_err(|source| Error::KubeError {
+            .map_err(|source| Error::KubeInferConfigError {
                 context: "create the kube client config".to_string(),
                 source,
             })?,
@@ -717,7 +731,7 @@ fn add_cells_for_cvs(row: &mut Vec<String>, oqty: &Option<Qty>, o100: &Option<Qt
             row.push(format!("{:.2}", f64::from(qty)));
             row.push(match o100 {
                 None => "".to_string(),
-                Some(q100) => format!("{:.0}%", qty.calc_percentage(&q100)),
+                Some(q100) => format!("{:.0}%", qty.calc_percentage(q100)),
             });
         }
     };
